unit MainForm;

{ * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
  * BTFileStreamDemo                                                            *
  *- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*
  * Copyright (c) 2002 - 2010 by Martin Offenwanger (www.dsplayer.de)  	    *
  *- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*
  * Mozilla Public License Version 1.1:                                     *
  *- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*
  * The contents of this file are used with permission, subject to the      *
  * Mozilla Public License Version 1.1 (the "License"); you may             *
  * not use this file except in compliance with the License. You may        *
  * obtain a copy of the License at                                         *
  * http://www.mozilla.org/MPL/MPL-1.1.html                                 *
  *                                                                         *
  * Software distributed under the License is distributed on an             *
  * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or          *
  * implied. See the License for the specific language governing            *
  * rights and limitations under the License.                               *
  * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * }
{
  @author(Martin Offenwanger: coder@dsplayer.de)
  @created(Dec 25, 2010)
  @lastmod(Dec 25, 2010)
  @supported compilers: Delphi 7 - DelphiXE
}

interface

uses
  // Runtime lib
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls, XPMan,
  // DSPlayer lib
  BTFileStream, BTStd;

type
  TFrmMain = class(TForm)
    BtnLoadPicture: TButton;
    Memo1: TMemo;
    ImgXmas: TImage;
    BtnClear: TButton;
    procedure BtnLoadPictureClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure BtnClearClick(Sender: TObject);
  private
    function DrawtoScreen(fp_Bitmap: PByte): Boolean;
    procedure Clear;
  end;

var
  FrmMain: TFrmMain;

implementation

{$R *.dfm}

procedure TFrmMain.BtnLoadPictureClick(Sender: TObject);
var
  l_FileStream: TBTFileStream;
  l_Result: TBTCommonError;
  lp_Buffer: PByte;
  l_Size: Int64;
  l_BytesRead: Cardinal;
begin
  lp_Buffer := nil;
  // create the filestram instance
  l_FileStream := TBTFileStream.Create;
  try
    // open a file
    l_Result := l_FileStream.OpenFile(
      IncludeTrailingPathDelimiter(ExtractFilePath(ParamStr(0))) + 'Xmas.bmp',
      btFmOpenRead,       // open file for read access
      btFmpOpenExisting,  // open a existing file
      btfmShareNone       // during file is open do not not share the file
      );
    if BTFailed(l_Result) then begin
      ShowMessage(l_FileStream.GetErrorString(l_Result));
      Abort;
    end;

    // get filesize
    l_Result := l_FileStream.GetSize(l_Size);
    if BTFailed(l_Result) then begin
      ShowMessage(l_FileStream.GetErrorString(l_Result));
      Abort;
    end;

    // get memory...
    try
      GetMem(lp_Buffer, l_Size);
    except
      ShowMessage('GetMem failed!');
      Abort;
    end;

    // read file data into the memory
    l_Result := l_FileStream.Read(lp_Buffer^, l_Size, l_BytesRead);
    if BTFailed(l_Result) then begin
      ShowMessage(l_FileStream.GetErrorString(l_Result));
      Abort;
    end;

    // draw the bitmap
    if not DrawtoScreen(lp_Buffer) then begin
      ShowMessage('Draw the Bitmap failed');
      Abort;
    end;

  finally
    // close file
    l_FileStream.CloseFile;
    // free filestream instance
    l_FileStream.Free;
    // free memory data
    if lp_Buffer <> nil then
      FreeMem(lp_Buffer, l_Size);
  end;

end;

function TFrmMain.DrawtoScreen(fp_Bitmap: PByte): Boolean;
var
  lp_BitmapInfoHeader: PBitmapInfoHeader;
  lp_data: PByte;
  l_i: Integer;
begin
  Result := False;
  // check if file is a windows bitmap
  if PBitmapFileHeader(fp_Bitmap).bfType <> $4D42 then
    Exit;
  // get pointer to bitmap bits
  lp_data := PByte(Cardinal(fp_Bitmap) +
    PBitmapFileHeader(fp_Bitmap).bfOffBits);
  // get bitmap info header
  lp_BitmapInfoHeader := PBitmapInfoHeader(Int64(fp_Bitmap) +
    SizeOf(TBitmapFileHeader));
  // check if format is 24 bit per pixel
  if lp_BitmapInfoHeader.biBitCount <> 24 then
    exit;
  // check if file is a uncompressed bitmap file..
  if lp_BitmapInfoHeader.biCompression <> BI_RGB then
    exit;
  // check if  no colortable is used in the file
  if (lp_BitmapInfoHeader.biClrUsed <> 0) or
    (lp_BitmapInfoHeader.biClrImportant <> 0) then
    exit;
  // check if bitmap is a bottom up bitmap..
  if lp_BitmapInfoHeader.biHeight < 0 then
    exit;
  // set pixel format
  ImgXmas.Picture.Bitmap.PixelFormat := pf24bit;
  // set size
  ImgXmas.Picture.Bitmap.Width := lp_BitmapInfoHeader.biWidth;
  ImgXmas.Picture.Bitmap.Height := lp_BitmapInfoHeader.biHeight;
  // draw picture lines...
  try
    for l_i := lp_BitmapInfoHeader.biHeight - 1 downto 0 do begin
      CopyMemory(ImgXmas.Picture.Bitmap.ScanLine[l_i], lp_data,
        lp_BitmapInfoHeader.biWidth * Round(lp_BitmapInfoHeader.biBitCount/8));
      inc(lp_data, lp_BitmapInfoHeader.biWidth *
         Round(lp_BitmapInfoHeader.biBitCount/8));
    end;
  except
    exit;
  end;
  // mark bitmap as modified
  ImgXmas.Picture.Bitmap.Modified := True;
  // all seems fine
  Result := True;
end;

procedure TFrmMain.Clear;
begin
  ImgXmas.Canvas.Brush.Color := clBlack;
  ImgXmas.Canvas.FillRect(ImgXmas.ClientRect);
end;

procedure TFrmMain.FormCreate(Sender: TObject);
begin
  Clear;
end;

procedure TFrmMain.BtnClearClick(Sender: TObject);
begin
  Clear;
end;

end.
